# Sparse lower-triangular matrix

#!# Unfinished work

module Algebra
class SparseMatrixLT
	attr_reader :m, :n # n == m

	# These options are disabled by default
	# When enabled, the indexes are ordered automatically
	# so that the object behaves like a symmetric matrix on read/write
	attr_accessor :write_symmetric
	attr_accessor :read_symmetric

	#=======================================================================
	# Constructors
	#=======================================================================
	
	def initialize(m)
		# Creates an empty matrix
		raise(TypeError,"Matrix dimension m must be larger than 0") if m <= 0
		@m = m
		@n = m
		@rows = {}
		@write_symmetric = false
		@read_symmetric = false

		@default_value = 0.0
	end

	#=======================================================================
	# Private - object options
	#=======================================================================
	
	def check_indexes(i,j, symmetric = false)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		raise(IndexError,"Col index (j) out of bounds") if j < 0
		if symmetric
			raise(IndexError,"Row index (j) out of bounds") if j >= @n
		else
			raise(IndexError,"Col index (j) out of bounds, j should be <= i") if j > i 
		end
	end
	
	#=======================================================================
	# Element access
	#=======================================================================

	def []=(i,j,value)
		# Check i,j
		check_indexes(i,j, @write_symmetric)
		# Symmetry
		if @write_symmetric
			if j > i
				i,j = j,i
			end
		end
		# Check/fix type
		raise(TypeError,"Only numbers can be stored in matrix") unless value.kind_of?(Numeric)
		value = value.to_f unless value.kind_of?(Float)
		# Erase element instead of setting it to zero
		if value == @default_value
			return @default_value unless @rows[i] # Return if zero was to be written into new row
			return @default_value unless @rows[i].has_key?(j)
			@rows[i].delete(j) # Remove element j
			@rows.delete(i) if @rows[i].empty? # Remove row when it stores no values
			return @default_value
		end
		# Create new row if needed
		# It is a hash that returns @default_value if the key does not exist
		@rows[i] = Hash.new(@default_value) unless @rows[i]
		# Save value
		@rows[i][j] = value
	end

	def [](i,j)
		# Check i,j
		check_indexes(i,j, @read_symmetric)
		# Symmetry
		if @write_symmetric
			if j > i
				i,j = j,i
			end
		end
		# Zero if row does not exist
		return @default_value unless @rows[i]
		return @rows[i][j]
	end

	def nonzero?(i,j)
		# Check i,j
		check_indexes(i,j, @read_symmetric)
		# Symmetry
		if @write_symmetric
			if j > i
				i,j = j,i
			end
		end
		return false unless @rows[i]
		return false unless @rows[i].has_key?(j)
		return true
	end

	def nonzero_count
		count = 0
		@rows.each_value{|row| count += row.size}
		return count
	end

	#=======================================================================
	# Row access
	#=======================================================================
	
	def row_as_array(i)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		a = Array.new(i+1,@default_value)
		row_each_nonzero_with_index(i){|x,j| a[j] = x}
		return a
	end

	#=======================================================================
	# Row iterators
	#=======================================================================

	def row_each_nonzero_with_index(i)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		return nil unless @rows[i]
		@rows[i].each_pair{|j, value|
			yield(value,j)
		}
		return nil
	end

	#=======================================================================
	# Iterators
	#=======================================================================

	def each_nonzero_with_index
		@rows.each_pair{|i,row|
			row.each_pair{|j,value|
				yield(value,i,j)
			}
		}
		return nil
	end

	#=======================================================================
	# Printing
	#=======================================================================
	
	def to_s(format="%10.6f", separator = " ", decoration_s = "| ", decoration_e = " |")
		#: Converts matrix to string. Format parameter can be used to modify
		#: the numbers.
		s = ''
		rows = []
		formatwidth = format.gsub(/\..*/,"").gsub("%",'').to_i

		self.m.times {|i|
			line = row_as_array(i).collect{|f| 
				f = 0.0 if f == -0.0
				sprintf(format,f)
			}
			(@m - i - 1).times{line << (" " * formatwidth)}
			rows << decoration_s + line.join(separator) + decoration_e
		}

		return rows.join("\n")
	end

	def inspect(format="%5.2f")
		#: Equal to -->to_s with prepended newline for nice irb output.
		#: Default format is shorter for convenience.
		return "\n" + to_s(format)
	end

	#=======================================================================
	# Comparison: not implemented
	#=======================================================================
	
	def ==(x)
		raise(NoMethodError, "Sparse matrix comparison not implemented")
	end

	def =~(x)
		raise(NoMethodError, "Sparse matrix comparison not implemented")
	end
end
end

__END__

	#=======================================================================
	# Row access
	#=======================================================================

	def row_nonzero_count(i)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		return 0 unless @rows[i]
		return @rows[i].size
	end


	def row_nonzero_as_array(i)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		return [] unless @rows[i]
		return @rows[i].values
	end

	def row_nonzero_indices_array(i)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		return [] unless @rows[i]
		return @rows[i].keys
	end

	#=======================================================================
	# Row iterators
	#=======================================================================

	def row_as_vector(i)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		a = Vector.zero(n)
		row_each_nonzero_with_index(i){|x,j| a[j] = x}
		return a
	end


	def row_each_nonzero_index(i)
		raise(IndexError,"Row index (i) out of bounds") if i >= @m || i < 0
		return nil unless @rows[i]
		@rows[i].each_key{|j|
			yield(j)
		}
		return nil
	end


	def each_nonzero_index
		@rows.each_pair{|i,row|
			row.each_key{|j|
				yield(i,j)
			}
		}
		return nil
	end

	#=======================================================================
	# Operators
	#=======================================================================
	
	def matrix_vector_multiply(operand)
		raise(TypeError, "Argument is not a vector") if operand.class != Vector
		raise(DimensionError,"Cant multiply matrix with vector of wrong size") unless n == operand.size
		a = []
		m.times{|i|
			sum = 0.0
			row_each_nonzero_with_index(i){|x,j|
				sum += x * operand[j]
			}
			a[i] = sum
		}
		return Vector.from_array(a)
	end

	def comutative_multiply(operand)
		#: Multiplication of matrix by: Float, Integer
		if operand.kind_of?(Numeric)
			result = SparseMatrix.new(m,n)
			each_nonzero_with_index{|x,i,j| result[i,j] = x * operand}
			return result
		else
			raise(TypeError,"Matrix.*: Can't comutatively multiply matrix by class #{operand.class}")
		end
	end

	#=======================================================================
	# Operators: matrix - something
	#=======================================================================
	
	def *(operand)
		#: Multiplication of matrix by: Float, Integer, Vector and Matrix
		if operand.kind_of?(Numeric)
			return self.comutative_multiply(operand)
		#--------------------------------------------------
		# elsif operand.kind_of?(Matrix)
		# 	return self.matrix_multiply(operand)
		#-------------------------------------------------- 
		elsif operand.kind_of?(Vector)
			return self.matrix_vector_multiply(operand)
		else
			raise(TypeError,"SparseMatrix.*: Can't multiply matrix by class #{operand.class}")
		end
	end

	#=======================================================================
	# Type conversion
	#=======================================================================

	def to_matrix
		m = Matrix.zero(@m,@n)
		each_nonzero_with_index{|x,i,j| m[i,j] = x}
		return m
	end

	def to_sparse_matrix_csx(row_mode)
		return SparseMatrixCSX.from_sparse_matrix(row_mode, self)
	end


end
end
