# Simple auxiliary class for working with molecular formulas

class MolecularFormula
	attr_reader :element_counts

	def initialize(counts = nil)
		if counts == nil
			@element_counts = {}
		else
			@element_counts = counts
		end
	end

	def MolecularFormula.from_geometry(geometry)
		return MolecularFormula.new(geometry.count_elements)
	end

	def MolecularFormula.from_s(string)
		# Simple, without detailed checks of the input
		# Reads both "H2O" and "H2O1"
		formula = MolecularFormula.new
		records = []
		rec = nil
		allow_downcase = false
		string.split("").each{|char|
			if char =~ /[A-Z]/
				# First letter of element - create new record
				rec = [char, ""]
				records << rec
				allow_downcase = true
			end
			if char =~ /[a-z]/
				# Error check
				unless allow_downcase
					raise "Element must start with capital letter"
				end
				# Subsequent letter
				rec[0] = rec[0] + char
			end
			if char =~ /[0-9]/
				rec[1] += char
				allow_downcase = false
			end
		}

		# Build element counts
		counts = {}
		records.each{|rec|
			name, num = rec
			num = "1" if num == ""
			counts[Atom.element_from_string(name)] = num.to_i
		}

		return MolecularFormula.new(counts)
	end

	def to_s(skip_1 = true)
		keys = @element_counts.keys.sort{|a,b| PeriodicTable.proton_number(a) <=> PeriodicTable.proton_number(b)}
		s = ""
		keys.each{|k|
			s += k.to_s
			s += @element_counts[k].to_s unless skip_1 && @element_counts[k] == 1
		}
		return s
	end

	def to_hash
		return @element_counts
	end
end
