#!/usr/bin/env ruby
$:.unshift('/home/rezac/cuby4')
require "classes/cuby.rb" # Load Cuby classes and libraries
require "classes/core/cuby_loader.rb"
require "classes/geometry/hybridization.rb"

g = Geometry.from_file(ARGV[0])

hyb = Hybridization.new(g)
hyb.evaluate_grimme_mod(:PA2009, true, true)

puts "hybridization"
puts hyb
puts

puts "analytical derivatives in r"
puts hyb.gradient
puts

# numerical gradient
# For each atom, calculate dCN/dx,y,z
step = 0.0001
numgrad = Matrix.zero(g.size)
step = g.each_with_index{|atom, i|
	cart_deriv = Coordinate.new
	3.times{|c|
		atom[c] += step
		hyb = Hybridization.new(g)
		hyb.evaluate_grimme_mod(:PA2009, true)
		hplus = hyb[i]
		atom[c] -= step * 2
		hyb = Hybridization.new(g)
		hyb.evaluate_grimme_mod(:PA2009, true)
		hminus = hyb[i]
		atom[c] += step
		
		cart_deriv[c] = (hplus - hminus) / 2.0 / step
	}
	g.each_with_index{|atom2, j|
		next if i == j
		vec = (atom-atom2)
		vec = vec / vec.abs
		numgrad[i,j] += vec.dot(cart_deriv)

	}
}
#numgrad = (numgrad + numgrad.transpose)/2
puts "Numerical gradient:"
puts numgrad

