require "classes/grids/spherical_grid_spiral"

class MolecularSurfaceGrid
	attr_reader :coords

	def initialize(geometry, n_or_dens, vdw_scale = 1.0, offset = 0.0, mode = :fixed_n)
		@coords = []

		# Precalculate scaled vdW radii
		radii = []
		geometry.each_index{|i|
			radii[i] = PeriodicTable.vdw_radius(geometry[i].element) * vdw_scale + offset
		}

		geometry.each_index{|i|
			atom = geometry[i]
			case mode
			when :fixed_n
				n = n_or_dens
			when :density
				surf = 4.0 * Math::PI * radii[i]**2
				n = (surf * n_or_dens).round
			else
				raise "mode should be either :fixed_n or :density"
			end
			atomgrid = SphericalGridSpiral.new(radii[i], n)
			atomgrid.iterate_xyz(atom){|crd|
				save = true
				geometry.each_index{|j|
					next if i == j
					if crd.distance(geometry[j]) < radii[j]
						save = false
						break
					end
				}
				@coords << crd if save
			}
		}
	end
end

