#===============================================================================
#===============================================================================


class PairwiseElementParameters

	def initialize(settings, keyword_name)
		hash = settings[keyword_name]
		@keyword_name = keyword_name
		@params = {}
		hash.each_pair{|element, hash2|
			begin
				ele1 = Atom.element_from_string(element)
			rescue
				Cuby::error "Unknown element encountered in keyword #{keyword_name}: #{element}"
			end

			@params[ele1] = {}
			unless hash2.class == Hash
				Cuby::error "Keyword #{keyword_name} should be a hash of hashes"
			end
			hash2.each_pair{|element2, value|
				begin
					ele2 = Atom.element_from_string(element2)
				rescue
					Cuby::error "Unknown element encountered in keyword #{keyword_name}: #{element2}"
				end
				@params[ele1][ele2] = value
			}

		}
	end

	def check_elements(geometry)
		# Check if the parameters cover all pairs of elements in the geometry
		elements = geometry.elements_in_system
		elements.each{|i|
			elements.each{|j|
				self[i,j]
			}
		}
	end

	def [](i,j)
		# Return the value for pair i-j
		if @params[i] && @params[i][j]
			return @params[i][j]
		elsif @params[j] && @params[j][i]
			return @params[j][i]
		else
			Cuby::error "Keyword #{@keyword_name}: missing value for pair #{i}-#{j}"
		end
	end

	def each_value
		# Iterate over values
		@params.each_pair{|i,hash2|
			hash2.each_pair{|key, value|
				yield(value)
			}
		}
		return nil
	end

	def map!
		# Modify values in place
		@params.each_pair{|i,hash2|
			hash2.each_pair{|key, value|
				hash2[key] = yield(value)
			}
		}
		return nil
	end
end
