#!/usr/bin/env ruby
$:.unshift(File.dirname(`which cuby4`)).uniq! # Magic line to add path to the executable to $RUBYLIB
require "yaml"
require "protocols/dataset/classes/data_set"
require "classes/cuby.rb"
Cuby.load_framework



unless ARGV.size == 2
  puts
  puts "Usage:"
  puts "dataset_ene_to_geo dataset_name geom_directory"
  puts
  exit
end

# Load dataset
dataset_path = File.dirname(`which cuby4`)+"/data/datasets"
filename = ARGV[0].downcase + ".yaml"
f = File.open(dataset_path+"/"+filename)
set = YAML.unsafe_load(f)
f.close

# Read geodb file
geodb = set.items.first.geometry.gsub(/:.*/,'')
f = File.open(File.dirname(`which cuby4`)+"/data/geometries/#{geodb}.yaml")
geodict = YAML.unsafe_load(f)
f.close

set.items.each{|item|
  geodb, geoname = item.geometry.split(":")
  geoname = geoname.to_i if geoname =~ /^[0-9]+$/
  geofile = geodict[geoname]
  lines = IO.readlines(ARGV[1] + "/" + geofile)
  if geofile =~ /\.xyz$/
    lines[1] = "reference_type=interaction_energy reference_unit=kcalmol reference_value=#{item.reference_value}"

    lines[1] += " charge=0 charge_a=0 charge_b=0"

    geo = Geometry.from_file(ARGV[1] + "/" + geofile)
    conn = Connectivity.new(geo)
    molecules = conn.molecules # as lists
    unless molecules.size == 2
      rasise "Not a dimer"
    end
    sel_a = GeometrySelections.atomlist_to_string(molecules[0],1)
    sel_b = GeometrySelections.atomlist_to_string(molecules[1],1)
    lines[1] += " selection_a=#{sel_a} selection_b=#{sel_b}"
    File.open(ARGV[1] + "/" + geofile, "w+"){|f|
      f.puts lines
    }
  end
}
