# Method-specific parametrs for D3 dispersion

require "classes/calculation/default_method_settings"
require "classes/misc/search_for_file.rb"

class D3Parameters < Hash

	def D3Parameters.load (settings, parameter_dir)
		# Load the parameters from files specified in settings

		parameters = D3Parameters.new
		
		# Make list of parameter files
		filenames = []
		settings[:d3_parameter_file].split(/\s*[\,,;,:]\s*/).each{|fn|
			filenames << SearchForFile.search(
				:filename => fn,
				:locations => ['./', parameter_dir, '/'],
				:extensions => ['', '.yaml']
			)
		}
		# At least one parameter file must be set up
		if filenames.size == 0
			Cuby::error("At least one parameter file must be provided in keyword 'd3_parameter_file'")
		end
		# Load each file
		filenames.each{|fn|
			f = File.open(fn)
			parameters.deep_merge!(YAML.unsafe_load(f))
			f.close
		}

		return parameters
	end

	def defaults_to_settings(settings)
		# Load method-specific defaults into settings using information from a parent calculation

		default_settings = DefaultMethodSettings.new(self)
		method_defaults_loaded = default_settings.find_and_merge(settings)

		#: Check whether all parameters needed for a given calculation are set
		case settings[:d3_damping]
		when :zero
			vars_to_check = [:d3_sr6, :d3_sr8, :d3_s6, :d3_s8, :d3_alpha6, :d3_alpha8]
		when :bj, :tt, :bjr0, :bjvdw
			vars_to_check = [:d3_s6, :d3_s8, :d3_a1, :d3_a2]
		when :erf
			vars_to_check = [:d3_s6, :d3_s8, :d3_a1, :d3_alpha6]
		when :tt_exp, :bj_exp
			vars_to_check = [:d3_s6, :d3_s8, :d3_a1, :d3_a2, :x0, :x1]
		when :op
			vars_to_check = [:d3_s6, :d3_s8, :d3_a1, :d3_a2, :d3_op_beta]
		when :cso
			vars_to_check = [:d3_s6, :d3_a1]
		when :none
			vars_to_check = [:d3_s6, :d3_s8]
		end
		vars_to_check << :d3_hh_para if settings[:d3_hh_fix]
		missing = []
		vars_to_check.each{|varname|
			missing << varname.to_s unless settings.set?(varname) || Settings.keyword(varname).when_not_present != :die
		}
		if missing.size > 0
			if method_defaults_loaded
				Cuby::error("Dispersion3 parameters missing although defaults were loaded for the present method:\n#{missing.join(', ')}")
			else
				Cuby::error("Dispersion3 parameters missing:\n#{missing.join(', ')}")
			end
		end
	end

end
