#!/usr/bin/ruby

require "yaml"

NAME_DF = {
	:bj => "Becke-Johnson", 
	:zero => "Zero", 
	:op => "Optimized power", 
	:cso => "C-six-only"
}

REF_DF = {
	:bj => "Grimme, S.; Ehrlich, S.; Goerigk, L. J. Comput. Chem. 2011, 32, 1456-1465.", 
	:zero => "Grimme, S.; Antony, J.; Ehrlich, S.; Krieg, H. J. Chem. Phys. 2010, 132, 19.", 
	:op => "Witte, J.; Mardirossian, N.; Neaton, J. B.; Head-Gordon, M. J. Chem. Theory Comput. 2017, 13, 2043-2052.", 
	:cso => "Schroeder, H.; Creon, A.; Schwabe, T. J. Chem. Theory Comput. 2015, 11, 3163-3170."
}

DOI_DF = {
	:bj => "10.1002/jcc.21759",
	:zero => "10.1063/1.3382344",
	:op => "10.1021/acs.jctc.7b00176",
	:cso => "10.1021/acs.jctc.5b00400"
}

PARAMS_DF = {
	:bj => {
		:d3_a1 => "a1",
		:d3_a2 => "a2",
		:d3_s8 => "s8"
	},
	:zero => {
		:d3_sr6 => "sr6",
		:d3_s8 => "s8"
	}, 
	:op => {
		:d3_a1 => "a1",
		:d3_a2 => "a2",
		:d3_s8 => "s8",
		:d3_op_beta => "beta"
	},
	:cso => {
		:d3_a1 => "a1"
	},
}

FUNCTIONALS = {
	"blyp" => "B-LYP",
	"b3lyp" => "B3-LYP",
	"pbe" => "PBE",
	"pbe0" => "PBE0",
	"tpss" => "TPSS",
	"bp86" => "B-P86",
	"bp" => "B-P",
	"b97d" => "B97-D"
}



f = File.open("dispersion3_parameters.yaml")
db = YAML.unsafe_load(f)

[:bj, :zero, :op, :cso].each{|damping|
	puts "<h3>"
	puts "#{NAME_DF[damping]} damping"
	puts "</h3>"

	puts "<table class='table_d3_para'>"
	puts "<tr>"
	puts "<th>Functional</th>"
	PARAMS_DF[damping].each_pair{|key, parname|
		puts "<th>#{parname}</th>"
	}
	puts "</tr>"
	func_params = db['values'][:default]['values'][:dft]['values'][damping]['values']["DZVP-DFT"]['values']
	func_params.each_pair{|func, params|
		next if func == "bp86"
		puts "<tr>"
		puts "<td>#{FUNCTIONALS[func]}</td>"
		PARAMS_DF[damping].each_pair{|key, parname|
			puts "<td>#{params[key.to_s]}</td>"
		}
		puts "</tr>"
	}
	puts "</table>"
	puts ""
}
