from http.server import HTTPServer, BaseHTTPRequestHandler
import json
from io import BytesIO
import torch
from torchmdnet.models.model import load_model
import sys

# Use GPU if available
if torch.cuda.is_available(): 
    dev = "cuda:0" 
else: 
    dev = "cpu" 

dev="cpu"

device = torch.device(dev)

def energy(elem,geom,model):
    types = torch.tensor(elem, dtype=torch.long)
    types = types.to(device)
    pos = torch.tensor(geom, dtype=torch.float32)
    pos = pos.to(device)
    energy, forces = model.forward(types, pos)#,batch)
    return (energy.item(),forces.tolist())

class SimpleHTTPRequestHandler(BaseHTTPRequestHandler):

    # Model is a class variable
    mod = None
    model_file = ""

    def do_POST(self):
        response=BytesIO()
        self.send_response(200)
        self.end_headers()
        content_length = int(self.headers['Content-Length'])
        body = self.rfile.read(content_length)
        data=json.loads(body)
        if(data[0]=="ping"):
            body=bytes(json.dumps('OK'),'ascii')
        if(data[0]=="load"):
            if self.__class__.model_file != data[1]:
                self.__class__.mod = load_model(data[1], derivative=True)
                self.__class__.mod = self.__class__.mod.to(device)
                self.__class__.model_file = data[1]
                #print(f"Model loaded: {data[1]}")
        if(data[0]=="geom"):
            #print("Compute")
            mol=data[1]
            ene=energy(mol[0],mol[1],self.__class__.mod)
            body=bytes(json.dumps(ene),'ascii')
        response.write(body)
        self.wfile.write(response.getvalue())

    # Disable server messages
    def log_message(self, format, *args):
        return

port = int(sys.argv[1])
#print(f"Starting calculation server at port {port}")
httpd = HTTPServer(('localhost', port), SimpleHTTPRequestHandler)
httpd.serve_forever()
