module ActionsAtomtype

	def atomtype_print
		# No geometry is written
		@write_geo = false

		# Build connectivity
		connectivity = Connectivity.new(@geometry, @settings[:connectivity_tolerance])

		# Atom types
		atomtypes = connectivity.atom_types_simple(@geometry)

		@geometry.each_with_index{|atom, i|
			printf("%10d%4s   ", i+1, atom.element)
			puts atomtypes[i]
		}

	end

	def atomtype_bonds
		# No geometry is written
		@write_geo = false

		# Build connectivity
		connectivity = Connectivity.new(@geometry, @settings[:connectivity_tolerance])

		# Atom types
		atomtypes = connectivity.atom_types_simple(@geometry)

		# Traverse bonds
		connectivity.each_bond{|i,j|
			r = @geometry[i].distance(@geometry[j])
			t1, t2 = [atomtypes[i], atomtypes[j]].sort
			e1 = t1.gsub(/:.*/, '')
			e2 = t2.gsub(/:.*/, '')
			printf("%-4s%-4s%-20s%-20s%12.4f%10d%10d\n", e1,e2,t1, t2, r, i+1,j+1)
		}
	end

	def atomtype_angles
		# No geometry is written
		@write_geo = false

		# Build connectivity
		connectivity = Connectivity.new(@geometry, @settings[:connectivity_tolerance])

		# Atom types
		atomtypes = connectivity.atom_types_simple(@geometry)

		# Traverse bonds
		connectivity.each_angle{|i,j,k|
			a = Coordinate.angle(@geometry[i], @geometry[j], @geometry[k])
			a *= 180.0 / Math::PI
			t1, t3 = [atomtypes[i], atomtypes[k]].sort
			unless t1 == atomtypes[i] && t3 == atomtypes[k]
				# Swapped, swap indices as well
				i,k = k,i
			end
			t2 = atomtypes[j]
			e1 = t1.gsub(/:.*/, '')
			e2 = t2.gsub(/:.*/, '')
			e3 = t3.gsub(/:.*/, '')
			printf("%-4s%-4s%-4s%-20s%-20s%-20s%12.4f%6d%6d%6d\n", e1,e2,e3, t1,t2,t3, a, i+1, j+1, k+1)
		}
	end
end
