module ActionsNoncovalent
	def list_nc_hbonds
		@write_geo = false # This action does not write geometry file

		hb_ele_d = [:O, :N, :C, :S]
		hb_ele_a = [:O, :N, :S]

		conn = Connectivity.new(@geometry, @settings[:connectivity_tolerance])
		molecules = conn.molecules
		geoms = conn.molecules.map{|list| @geometry.geometry_from_list(list) }

		if geoms.size != 2
			@messages << "not_dimer(#{geoms.size})"
			return
		end
		g1, g2 = geoms

		# XH-Y bond
		hbonds = []
		lengths = []
		@geometry.each_with_index{|ax, ix|
			next unless hb_ele_d.include?(ax.element)
			conn.bound_atoms(ix).each{|ih|
				ah = @geometry[ih]
				next unless ah.element == :H
				@geometry.each_with_index{|ay, iy|
					next unless hb_ele_a.include?(ay.element)
					next if g1.include?(ax) && g1.include?(ay)
					next if g2.include?(ax) && g2.include?(ay)
					# Now, a possible h-bond
					dist = ah.distance(ay)
					vdw = PeriodicTable.vdw_radius(:H) + PeriodicTable.vdw_radius(ay.element)
					next if dist > vdw * @settings[:geometry_nc_vdwlimit]
					angle = Coordinate.angle(ax,ah,ay) / Math::PI * 180
					next if angle < 120
					hbonds << "#{ax.element}#{ah.element}-#{ay.element}"
					lengths << dist
				}
			}
		}

		if @settings[:geometry_nc_shortest]
			min = lengths.min
			hb2 = []
			len2 = []
			hbonds.each_index{|i|
				if lengths[i] < min * 1.025 # 2.5% tolerance
					hb2 << hbonds[i]
					len2 << lengths[i]
				end
			}
			hbonds = hb2
			lengths = len2
		end

		if hbonds.size == 0
			@messages << "---"
		else
			@messages << hbonds.join(",")
		end
	end

	def list_nc_sigmahole_bonds
		@write_geo = false # This action does not write geometry file

		hole_elements = [:S, :P, :Cl, :Br, :I]
		neg_elements = [:N, :O, :F]

		conn = Connectivity.new(@geometry, @settings[:connectivity_tolerance])
		molecules = conn.molecules
		geoms = conn.molecules.map{|list| @geometry.geometry_from_list(list) }

		if geoms.size != 2
			@messages << "not_dimer(#{geoms.size})"
			return
		end
		g1, g2 = geoms

		xbonds = []
		lengths = []
		@geometry.each_with_index{|ax, ix|
			next unless hole_elements.include?(ax.element)
			@geometry.each_with_index{|ay, iy|
				next unless neg_elements.include?(ay.element)
				dist = ax.distance(ay)
				vdw = PeriodicTable.vdw_radius(ax.element) + PeriodicTable.vdw_radius(ay.element)
				next if dist > vdw * @settings[:geometry_nc_vdwlimit]
				# Evaluate angles
				min_angle = 999.9
				conn.bound_atoms(ix).each{|i|
					angle = Coordinate.angle(@geometry[i], ax, ay) / Math::PI * 180
					min_angle = angle if min_angle < min_angle
				}
				next if min_angle < 180 - 30
				xbonds << "#{ax.element}-#{ay.element}"
				lengths << dist
			}
		}

		if xbonds.size == 0
			@messages << "---"
		else
			@messages << xbonds.join(",")
		end
	end

	def list_nc_closest_vdw
		@write_geo = false # This action does not write geometry file

		conn = Connectivity.new(@geometry, @settings[:connectivity_tolerance])
		molecules = conn.molecules

		if molecules.size != 2
			@messages << "not_dimer(#{molecules.size})"
			return
		end

		mod_radii = @settings.elements_hash(:geometry_custom_vdw_radii)

		min = 9.9e99
		min_i = nil
		min_j = nil

		molecules[0].each{|i|
			molecules[1].each{|j|
				d = @geometry[i].distance(@geometry[j])
				vdw = PeriodicTable.vdw_radius(@geometry[i].element, :default, mod_radii) + 
				      PeriodicTable.vdw_radius(@geometry[j].element, :default, mod_radii)
				rd = d / vdw
				if rd < min
					min = rd
					min_i = i
					min_j = j
				end
			}
		}

		@messages << sprintf("%-6d%-6d%-6s%8.3f%8.3f%\n",
		       min_i+1,
		       min_j+1,
		       @geometry[min_i].element.to_s + "-" + @geometry[min_j].element.to_s,
		       @geometry[min_i].distance(@geometry[min_j]),
		       min * 100
		)
	end
end
