require "classes/geometry/geometry_series.rb"

module ActionsSeries

	def expand_series
		# Load and check geometries
		load_series

		@geometries.each_index{|i|
			filename = eval(@settings[:expand_series_template])
			@messages << "Frame #{i} written to file '#{filename}'"
			@geometries[i].write_xyz(:file => filename)
		}

		# Do not write the geometry, it is handled here
		@write_geo = false
	end

	def series_energy_minimum
		# Load and check geometries
		load_series

		# Read energies from the geometries
		energies = []
		@geometries.each_index{|i|
			if ene = /energy\s*=* *(\S*)/i.match(@geometries[i].info[:xyz_remark])[1]
				energies << ene.to_f
			end
		}
		# Check that we have all energies
		unless energies.size == @geometries.size
			Cuby::error("No energies found in the geometry file")
		end
		# Find minimum
		min = 99.9e99
		min_i = nil
		energies.each_index{|i|
			if energies[i] < min
				min = energies[i]
				min_i = i
			end
		}
		# Save the minimum as the geometry
		@messages << "Frame #{min_i} has the lowest energy #{min}"
		@geometry = @geometries[min_i]
		# Save the original comment
		@geometry.info[:protocol_comment] = @geometries[min_i].info[:xyz_remark]
	end

	def load_series
		# Check whether geometry is an .xyz file
		unless FileTest.file?(File.expand_path(@settings[:geometry]))
			Cuby::error("For reading multiple geometries, only .xyz files are supported")
		end

		@geometries = GeometrySeries.from_settings(settings, "scan:")
		@geometries.size_check
	end
end
