
module ProtocolNativeOpt
	#=======================================================================
	# Protocol header
	#=======================================================================
	PROTOCOL_TYPE = :elementary_calculation
	#=======================================================================

	def print_header_protocol
		# Print header
		Cuby.log.print_cuby_logo(:comment => false, :text => 'Native optimization')
	end

	def prepare_protocol
		# Read geometry
		load_geometry
		# Create calculation
		@calculation = Calculation.new(@name, @settings, @geometry)
		# Prepare calculation
		what = [:energy, :native_opt]
		@calculation.prepare(what)
	end

	def run_protocol(queue)
		# Submit calculation
		@calculation.send_to_queue(queue)
	end

	def print_protocol
		# Print results
		results.print_energy
		Cuby::log.puts_v_only(:normal,"")
		# Save geometry
		if @settings.set?(:restart_file)
			unless @settings[:restart_file] == ""
				filename = @settings[:restart_file]
				filetype = GeoFile.type_from_filename(filename)
				if filetype == :xyz
					s = "Energy #{@calculation.results.energy} kcal/mol"
					s += " Converged!" if @opt_status == :converged
					@geometry.write_file(filename, filetype, {:second_line => s, :append => @settings[:restart_file_append]})
				else
					@geometry.write_file(filename, filetype, {
						:append => @settings[:restart_file_append],
						:extra_columns => @settings[:pdb_extra_columns]
					})
				end
				Cuby::log.puts_v_only(:normal,"Optimized geometry written to file #{filename}")
			end
		else
			s = "Energy #{@calculation.results.energy} kcal/mol"
			s += " Converged!"
			@geometry.write_xyz(:file => "optimized.xyz", :second_line => s, :append => @settings[:restart_file_append])
			Cuby::log.puts_v_only(:normal,"Optimized geometry written to file optimized.xyz")
		end
		Cuby::log.puts_v_only(:normal,"")
	end

	def cleanup_protocol
		@calculation.cleanup
	end

	def results
		# Results are collected only first time they are needed
		unless @results
			# Get results
			@results = @calculation.results
		end
		return @results
	end

	#=======================================================================
	# Optional protocol functionality
	#=======================================================================
	
	def single_line_result
		return sprintf("Energy: %20.6f kcal/mol", results.energy)
	end

	def single_number_result
		return results.energy
	end

	def update_geometry(geometry)
		# Update geometry, invalidate the results on previous one
		@geometry.copy_coordinates_from(geometry)
		@results = nil
	end

end
