#!/usr/bin/env ruby
$:.unshift(File.dirname(__FILE__)+"/..").uniq! # Magic line to add path to the executable to $RUBYLIB
require "classes/cuby.rb" # Load Cuby classes and libraries

ELEMENTS = [:O,:N,:S]

def find_structural_hbonds(geo, conn, threshold = 0.95, bad_cases = false)
	pairs = []
	pairs_no_h = []

	conn.each_nonbonded_above_1_4{|i,j|
		at_i = geo[i]
		at_j = geo[j]

		if ELEMENTS.include?(at_i.element) && ELEMENTS.include?(at_j.element)
			sum_radii = PeriodicTable.vdw_radius(at_i.element) +
				    PeriodicTable.vdw_radius(at_j.element)
			if at_i.distance(at_j) <= sum_radii * threshold
				pairs << [i,j]
				# Check if there is H along the bond
				nearby = conn.bound_atoms(i) | conn.bound_atoms(j)
				ok = false
				nearby.each{|k|
					next unless geo[k].element == :H
					angle = [Coordinate.angle(at_i, at_j, geo[k]),
					         Coordinate.angle(at_j, at_i, geo[k])].max
					angle = angle / Math::PI * 180
					ok = true if angle < 30
				}
				pairs_no_h << [i,j] unless ok
			end
		end

	}

	return pairs_no_h if bad_cases
	return pairs

end

Cuby.application {
	Cuby.load_framework

	geo = Geometry.from_file(ARGV[0])
	conn = Connectivity.new(geo)

	pairs = find_structural_hbonds(geo, conn, 1.05, false)
	puts "Contacts with < 1.05 vdW distance:   #{'%3d' % pairs.size}, written to Hbonds_105.mol2"
	GeoFile::MOL2.write(geo, :bondlist => pairs, :do_not_rename_atoms => true, :file => "Hbonds_105.mol2")
	
	pairs = find_structural_hbonds(geo, conn, 0.95, false)
	puts "H-bonds with < 0.95 vdW distance:    #{'%3d' % pairs.size}, written to Hbonds_095.mol2"
	GeoFile::MOL2.write(geo, :bondlist => pairs, :do_not_rename_atoms => true, :file => "Hbonds_095.mol2")

	pairs = find_structural_hbonds(geo, conn, 0.95, true)
	puts "Out of that, with missing hydrogen:  #{'%3d' % pairs.size}, written to Hbonds_bad.mol2"
	GeoFile::MOL2.write(geo, :bondlist => pairs, :do_not_rename_atoms => true, :file => "Hbonds_bad.mol2")


}
