#!/usr/bin/env ruby

# This scripts format a table of additional reference data from a from spreadsheet to the
# format that can be pasted into the data set file.
# This script uses an extended format with references and metadata


# Print some help if no argument provided
if ARGV.size == 0
puts "
Usage:
dataset_alt_ref2.rb table.dat >> set.yaml

The table is organized by columns (one for each reference data set),
each column containing:
   set name,
   energy calculation description,
   notes,
   RMSE to reference,
   MUE to reference,
   MSE to reference,
   DOI,
   citation,
   value1,
   value2,
   ...
\n"
exit
end

HEADER_SIZE = 8

puts ""
puts "#==============================================================================="
puts "# Alternative data"
puts "#==============================================================================="
puts ""
puts "alternative_reference:"

table = IO.readlines(ARGV[0]).map{|line| line.strip.split("\t")}

table[0].size.times{|col|
	puts "  \"#{table[0][col]}\": !ruby/object:ProtocolDataset::DataSetReference"
	puts "    method_energy: \"#{table[1][col]}\""
	puts "    notes: \"#{table[2][col]}\"" if table[2][col]
	puts "    references:"
	puts "      \"#{table[6][col]}\": \"#{table[7][col]}\""
	puts "    errors:"
	puts "      rmse: #{table[3][col]}"
	puts "      mue: #{table[4][col]}"
	puts "      mse: #{table[5][col]}"
	# Collect values
	values = []
	i = HEADER_SIZE
	while i < table.size
		values << table[i][col]
		i += 1
	end
	puts "    values: [ #{values.join(', ')} ]"
	puts
}

