#!/usr/bin/env ruby
$:.unshift(File.dirname(__FILE__)+"/..").uniq! # Magic line to add path to the executable to $RUBYLIB
require "classes/cuby.rb"
Cuby.load_framework

require "classes/grids/tesselate_triangles.rb"

# generates a grid and associated triangles for surface visualization

if ARGV.size != 4
	$stderr.puts ""
	$stderr.puts "The program requires 4 parameters:"
	$stderr.puts "esp_grid_generate geometry_file vdw_scaling vdw_offset grid_quality"
	$stderr.puts ""
	$stderr.puts "Reasonable values are:"
	$stderr.puts "esp_grid_generate geometry_file 1.0 1.0 4"
	$stderr.puts ""
	exit
end

geofile = ARGV[0]
vdw_scale = ARGV[1].to_f # Default 1.0
offset = ARGV[2].to_f # Default 1.0
grid_order = ARGV[3].to_i # Default 4

geometry = Geometry.from_file(geofile)

#===============================================================================
# Triangular surface construction
#===============================================================================
radii = []
geometry.each_index{|i|
	radii[i] = PeriodicTable.vdw_radius(geometry[i].element) * vdw_scale + offset
}

spheres = []
geometry.each_index{|i|
	atom = geometry[i]
	order = grid_order
	#order -= 1 if atom.element == :H
	#order -= 1 if atom.element == :F && geometry.size > 2
	spheres[i] = TesselateTriangles.new(atom, radii[i], order)
}

# write grid
f = File.open("grid.dat", "w+")
ft = File.open("triangles.dat", "w+")
geometry.each_index{|i|
	spheres[i].triangles.each{|tri|
		crd = tri.center
		save = true
		geometry.each_index{|j|
			next if i == j
			if crd.distance(geometry[j]) < radii[j] - 0.1
				save = false
				break
			end
		}
		if save
			f.printf("%10.5f%10.5f%10.5f\n", crd.x, crd.y, crd.z)

			tricrds = []
			tri.each_point{|coord|
				tricrds << coord.to_a.map{|x| "%.4f" % x}
			}
			ft.puts tricrds.join("\t")
		end
	}
}
f.close
ft.close
